"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _helperModuleImports = require("@babel/helper-module-imports");

var _path = require("path");

var _package = _interopRequireDefault(require("../package.json"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// @ts-ignore
const filterFalsy = array => array.filter(Boolean);

var _default = ({
  types: t
}) => {
  const getObjectExpression = styleObject => {
    if (!Object.keys(styleObject).length) return;
    return t.objectExpression(Object.entries(styleObject).map(([key, value]) => t.objectProperty(t.identifier(key), typeof value === 'number' ? t.numericLiteral(value) : t.stringLiteral(value))));
  };

  const getClassNameExpression = nodeValue => {
    if (t.isStringLiteral(nodeValue)) return nodeValue;
    if (t.isJSXExpressionContainer(nodeValue)) return nodeValue.expression;
  };

  const createHelpers = (path, options = {}) => {
    const tailwindRNExportName = options.tailwindRNExportName || 'default';
    const tailwindRNExportPath = options.tailwindRNExportPath ? (0, _path.resolve)(options.tailwindRNExportPath) : 'tailwind-rn';

    const tailwind = require(tailwindRNExportPath)[tailwindRNExportName];

    const getTailwindExpression = classesExpression => {
      if (!classesExpression) return;

      if (t.isTemplateLiteral(classesExpression)) {
        const styleObject = tailwind(classesExpression.quasis.map(templateElement => templateElement.value.raw.trim()).filter(Boolean).join(' '));
        const templateParamsExpressions = classesExpression.expressions;
        return filterFalsy([getObjectExpression(styleObject), ...templateParamsExpressions.flatMap(getTailwindExpression)]);
      }

      if (t.isStringLiteral(classesExpression)) {
        const styleObject = tailwind(classesExpression.value);
        return filterFalsy([getObjectExpression(styleObject)]);
      }

      const tailwindIdentifier = (0, _helperModuleImports.addNamed)(path, tailwindRNExportName, tailwindRNExportPath);
      return [t.callExpression(tailwindIdentifier, [classesExpression])];
    };

    const getStyleExpressionAndRemoveStyleNode = () => {
      if (!Array.isArray(path.container)) return;
      const container = path.container;
      const styleNodeIndex = container.findIndex(({
        name
      }) => name.name === 'style');
      if (styleNodeIndex === -1) return;
      const styleNodeValue = container[styleNodeIndex].value;
      if (!t.isJSXExpressionContainer(styleNodeValue)) return;
      const styleExpression = styleNodeValue.expression;
      path.getSibling(styleNodeIndex).remove();
      return styleExpression;
    };

    return {
      getTailwindExpression,
      getStyleExpressionAndRemoveStyleNode
    };
  };

  return {
    name: _package.default.name,
    visitor: {
      JSXAttribute(path, {
        opts
      }) {
        const {
          node
        } = path;
        if (node.name.name !== 'className') return;
        const {
          getStyleExpressionAndRemoveStyleNode,
          getTailwindExpression
        } = createHelpers(path, opts);
        const styleExpression = getStyleExpressionAndRemoveStyleNode();
        const classesExpression = getClassNameExpression(node.value);
        const tailwindExpression = getTailwindExpression(classesExpression);

        if (!tailwindExpression || !tailwindExpression.length) {
          path.remove();
          return;
        }

        node.name.name = 'style';
        node.value = t.jSXExpressionContainer(t.arrayExpression(filterFalsy([...tailwindExpression, styleExpression])));
      }

    }
  };
};

exports.default = _default;